#ifndef CONNECTION_MANAGER_H
#define CONNECTION_MANAGER_H

#include <Instrument.h>
#include <Configuration.h>

#include <tango.h>

#include <map>
#include <sstream>
#include <stdexcept>

#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>

#include <soci/connection-pool.h>
#include <soci/session.h>

namespace FitsImporter_ns
{

class ConnectionManager : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//	[Public] shared pointer typedef	
//------------------------------------------------------------------------------
	typedef boost::shared_ptr<ConnectionManager> SP;
	typedef boost::shared_ptr<soci::session> SessionSP;

protected:
//------------------------------------------------------------------------------
//	[Protected] constructor destructor deleter
//------------------------------------------------------------------------------
	ConnectionManager(Tango::DeviceImpl*, Configuration::SP, Instrument::SPVector)
		throw(std::runtime_error);
	virtual ~ConnectionManager();

	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(ConnectionManager* c) { delete c; }
	};

public:
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------
	static ConnectionManager::SP create(Tango::DeviceImpl*, Configuration::SP,
        Instrument::SPVector) throw(std::runtime_error);

	SessionSP getSession(Destination::SP) throw(std::runtime_error);

protected:
//------------------------------------------------------------------------------
//	[Protected] Connection pool class
//------------------------------------------------------------------------------
	class PoolID
	{
	public:
		PoolID(std::string host, int port, std::string user) :
			m_host(host), m_port(port), m_user(user) {}

		std::string getHost() const { return m_host; }
		int getPort() const { return m_port; }
		std::string getUser()const { return m_user; }

	bool operator<(const PoolID& id) const
	{
		std::stringstream str_1;
		str_1 << m_host << m_port << m_user;

		std::stringstream str_2;
		str_2 << id.getHost() << id.getPort() << id.getUser();

		return str_1.str().compare( str_2.str() );
	}

	private:
		std::string m_host;
		int m_port;	
		std::string m_user;	
	};

//------------------------------------------------------------------------------
//	[Protected] shared pointer typedef	
//------------------------------------------------------------------------------    
	typedef boost::shared_ptr< soci::connection_pool > PoolSP;
    
//------------------------------------------------------------------------------
//	[Protected] Class variables
//------------------------------------------------------------------------------
	//Connection pool map
	std::map<PoolID, PoolSP> m_pool;
};

}   //End of namespace

#endif /*!CONNECTION_MANAGER_H*/
