#ifndef DMDB_IMPORTER_H
#define	DMDB_IMPORTER_H

#include <Instrument.h>

#include <tango.h>

#include <stdexcept>

#include <boost/shared_ptr.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/optional/optional.hpp>
#include <boost/scoped_ptr.hpp>

#include <soci/error.h>
#include <soci/session.h>

namespace FitsImporter_ns
{

class DMDBImporter : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//  [Public] Shared pointer typedef
//------------------------------------------------------------------------------
    typedef boost::shared_ptr<DMDBImporter> SP;

protected:
//------------------------------------------------------------------------------
//	[Protected] Constructor destructor deleter
//------------------------------------------------------------------------------
    DMDBImporter(Tango::DeviceImpl*, std::string, unsigned short, std::string,
        std::string, std::string, std::string, std::string, std::string);

    virtual ~DMDBImporter();

	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(DMDBImporter* d) { delete d; }
	};

public:
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------
    static DMDBImporter::SP create(Tango::DeviceImpl*, std::string,
        unsigned short, std::string, std::string, std::string, std::string,
        std::string, std::string);

    virtual void connect() throw(soci::soci_error);

    virtual bool isConnected();

    virtual void disconnect();

    virtual Instrument::SPVector retrieveInstrumentList(std::vector<std::string>&)
        throw(std::invalid_argument, soci::soci_error);

    virtual Instrument::SP retrieveInstrument(std::string)
        throw(std::invalid_argument, soci::soci_error);

protected:
//------------------------------------------------------------------------------
//  [Protected] Utilities methods
//------------------------------------------------------------------------------
    virtual Instrument::SP getInstrument(std::string)
        throw(soci::soci_error);

    virtual Destination::SP getDestination(int)
        throw(soci::soci_error);

    virtual Mapping::SPVector getMapping(int)
        throw(soci::soci_error);

//------------------------------------------------------------------------------
//  [Protected] Instrument tuple typedef
//------------------------------------------------------------------------------
    typedef boost::tuple< boost::optional<int>, boost::optional<std::string>, boost::optional<int>, 
            boost::optional<std::string>, boost::optional<std::string>, boost::optional<int>,
            boost::optional<std::string>, boost::optional<int> > InstrumentTuple;

    typedef std::vector< InstrumentTuple > InstrumentTupleVector;

//------------------------------------------------------------------------------
//  [Protected] Destination tuple typedef
//------------------------------------------------------------------------------
    typedef boost::tuple< boost::optional<int>, boost::optional<std::string>,
            boost::optional<int>, boost::optional<std::string>,
            boost::optional<std::string>, boost::optional<std::string>,
            boost::optional<std::string>, boost::optional<std::string>,
            boost::optional<std::string> > DestinationTuple;

    typedef std::vector< DestinationTuple > DestinationTupleVector;

//------------------------------------------------------------------------------
//  [Protected] Mapping tuple typedef
//------------------------------------------------------------------------------
    typedef boost::tuple< boost::optional<int>, boost::optional<std::string>,
            boost::optional<std::string>, boost::optional<int>,
            boost::optional<std::string>, boost::optional<std::string>,
            boost::optional<int>  > MappingTuple;

    typedef std::vector< MappingTuple > MappingTupleVector;

//------------------------------------------------------------------------------
//	[Protected] Class variables
//------------------------------------------------------------------------------
    //Data model database host
    std::string m_host;

    //Data model database port
    unsigned short m_port;

    //Data model database login user
    std::string m_user;

    //Data model database login password
    std::string m_password;

    //Data model schema name
    std::string m_schema;

    //Data model instrument table name
    std::string m_instrumentTable;

    //Data model destination table name
    std::string m_destinationTable;

    //Data model mapping table name
    std::string m_mappingTable;

    //Database connection scoped pointer
    boost::scoped_ptr<soci::session> m_session_sp;
};

}   //End of namespace

#endif	/* DMDB_IMPORTER_H */
