#ifndef DMDB_VERIFIER_H
#define	DMDB_VERIFIER_H

#include <ConnectionManager.h>
#include <Instrument.h>

#include <tango.h>

#include <boost/shared_ptr.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/optional/optional.hpp>

#include <soci/error.h>

namespace FitsImporter_ns
{

class DMDBVerifier : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//  [Public] Shared pointer typedef
//------------------------------------------------------------------------------
    typedef boost::shared_ptr<DMDBVerifier> SP;

protected:    
//------------------------------------------------------------------------------
//	[Protected] Constructor destructor deleter
//------------------------------------------------------------------------------
    DMDBVerifier(Tango::DeviceImpl*, ConnectionManager::SP);
        
    virtual ~DMDBVerifier();
    
	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(DMDBVerifier* d) { delete d; }
	};    

public:    
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------    
    static DMDBVerifier::SP create(Tango::DeviceImpl*, ConnectionManager::SP);
    
    virtual void testInstrumentListMapping(Instrument::SPVector)
        throw(soci::soci_error, std::runtime_error);
    
    virtual void testInstrumentMapping(Instrument::SP)
        throw(soci::soci_error, std::runtime_error);

protected:
//------------------------------------------------------------------------------
//  [Protected] Columns tuple typedef
//------------------------------------------------------------------------------    
    typedef boost::tuple< boost::optional<std::string>, 
        boost::optional<std::string>, boost::optional<std::string> > ColumnTuple;        
    
    typedef std::vector< ColumnTuple > ColumnTupleVector;
    
//------------------------------------------------------------------------------
//  [Protected] Utilities methods
//------------------------------------------------------------------------------
    virtual void testColumnDefinition(ColumnTupleVector&, 
        std::string, std::string, bool) throw(std::runtime_error);
    
    virtual bool isEqual(ColumnTuple, std::string)
        throw(std::runtime_error);
    
//------------------------------------------------------------------------------
//	[Protected] Class variables
//------------------------------------------------------------------------------
    //Connection manager shared pointer
    ConnectionManager::SP m_connectionManager_sp;
};

}   //End of namespace

#endif	/* DMDB_VERIFIER_H */

