#ifndef DESTINATION_H
#define DESTINATION_H

#include <iostream>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <Mapping.h>

namespace FitsImporter_ns
{

class Destination
{
public:
//------------------------------------------------------------------------------
//	[Public] Shared pointer typedef
//------------------------------------------------------------------------------
	typedef boost::shared_ptr<Destination> SP;
	typedef std::vector< Destination::SP > SPVector;

private:
//------------------------------------------------------------------------------
//	[Private] Constructor destructor deleter
//------------------------------------------------------------------------------
	Destination(std::string host, int port, std::string user,
		std::string password, std::string schema, std::string table,
		std::string storagePath, std::string dirName,
        std::vector< boost::shared_ptr<Mapping> >& mapping_spvector) :
		m_host(host), m_port(port), m_user(user), m_password(password),
		m_schema(schema), m_table(table), m_storagePath(storagePath),
		m_dirName(dirName), m_mapping_spvector(mapping_spvector) {}
	~Destination() {}

	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(Destination* d) { delete d; }
	};

public:
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------
	static Destination::SP create(std::string host, int port,
		std::string user, std::string password, std::string schema,
		std::string table, std::string storagePath, std::string dirName,
		Mapping::SPVector& mapping_spvector)
	{
		Destination::SP d_sp( new Destination(host, port, user, password,
			schema, table, storagePath, dirName, mapping_spvector),
			Destination::Deleter() );
		return d_sp;
	}

	std::string getHost() const { return m_host; }
	int getPort() const { return m_port; }
	std::string getUser() const { return m_user; }
	std::string getPassword() const { return m_password; }
	std::string getSchema() const { return m_schema; }
	std::string getTable() const { return m_table; }
	std::string getStoragePath() const { return m_storagePath; }
    std::string getDirName() const { return m_dirName; }
	Mapping::SPVector getMappingSPVector() const { return m_mapping_spvector; }

private:
//------------------------------------------------------------------------------
//	[Private] class variables
//------------------------------------------------------------------------------
	//Meta data database host
	const std::string m_host;

    //Meta data database port
	const int m_port;

	//Meta data database user
	const std::string m_user;

	//Meta data database user
	const std::string m_password;

	//Meta data database user
	const std::string m_schema;

	//Meta data database user
	const std::string m_table;

	//Meta data file storage path
	const std::string m_storagePath;

    //Meta data file directory name
    const std::string m_dirName;

	//Mapping vector
	const Mapping::SPVector m_mapping_spvector;
};

}   //End of namespace

#endif /*!DESTINATION_H*/
