/*----- PROTECTED REGION ID(FitsImporter.cpp) ENABLED START -----*/
//static const char *RcsId = "$Id:  $";
//=============================================================================
//
// file :        FitsImporter.cpp
//
// description : C++ source for the FitsImporter class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               FitsImporter are implemented in this file.
//
// project :     Fits importer device server
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
// $Author:  $
//
// $Revision:  $
// $Date:  $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================

#include <FitsImporter.h>
#include <FitsImporterClass.h>
#include <Configuration.h>
#include <DMDBImporter.h>
#include <Instrument.h>

#include <sys/inotify.h>

/*----- PROTECTED REGION END -----*/	//	FitsImporter.cpp

/**
 *  FitsImporter class description:
 *    Fits importer device server
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name  |  Method name
//================================================================
//  State         |  Inherited (no method)
//  Status        |  Inherited (no method)
//  On            |  on
//  Off           |  off
//  ResetCounter  |  reset_counter
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  RegularFileCounter  |  Tango::DevULong	Scalar
//  WarningFileCounter  |  Tango::DevULong	Scalar
//  ErrorFileCounter    |  Tango::DevULong	Scalar
//================================================================

namespace FitsImporter_ns
{
/*----- PROTECTED REGION ID(FitsImporter::namespace_starting) ENABLED START -----*/

//	static initializations

/*----- PROTECTED REGION END -----*/	//	FitsImporter::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::FitsImporter()
 *	Description : Constructors for a Tango device
 *                implementing the classFitsImporter
 */
//--------------------------------------------------------
FitsImporter::FitsImporter(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(FitsImporter::constructor_1) ENABLED START -----*/
	init_device();

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::constructor_1
}
//--------------------------------------------------------
FitsImporter::FitsImporter(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(FitsImporter::constructor_2) ENABLED START -----*/
	init_device();

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::constructor_2
}
//--------------------------------------------------------
FitsImporter::FitsImporter(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(FitsImporter::constructor_3) ENABLED START -----*/
	init_device();

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void FitsImporter::delete_device()
{
	DEBUG_STREAM << "FitsImporter::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::delete_device) ENABLED START -----*/

	//	Delete device allocated objects

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::delete_device
	delete[] attr_RegularFileCounter_read;
	delete[] attr_WarningFileCounter_read;
	delete[] attr_ErrorFileCounter_read;
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void FitsImporter::init_device()
{
	DEBUG_STREAM << "FitsImporter::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::init_device_before) ENABLED START -----*/

    set_state(Tango::INIT);
    set_status("Initializing device");

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::init_device_before


	//	Get the device properties from database
	get_device_property();

	attr_RegularFileCounter_read = new Tango::DevULong[1];
	attr_WarningFileCounter_read = new Tango::DevULong[1];
	attr_ErrorFileCounter_read = new Tango::DevULong[1];

	/*----- PROTECTED REGION ID(FitsImporter::init_device) ENABLED START -----*/

    //Initialize regular file counters to zero
	*attr_RegularFileCounter_read = 0;

    //Initialize warning file counters to zero
	*attr_WarningFileCounter_read = 0;

    //Initialize error file counters to zero
	*attr_ErrorFileCounter_read = 0;

    if(get_state() != Tango::FAULT)
    {
        try
        {
            //Create i-notify mask from event list property
            const uint32_t inotifyMask = create_inotify_mask(eventList);

            //Initialize configuration container class
            Configuration::SP m_configuration_sp = Configuration::create(watchPath,
                    workerNumber, sleepTime, waitTime, connectionNumber, inotifyMask);

            //Initialize data model manager using tango properties as parameters
            DMDBImporter::SP dMDBImporter = DMDBImporter::create(this, dMDBHost,
                dMDBPort, dMDBUser, dMDBPassword, dMDBSchema, dMDBInstrumentTable,
                dMDBDestinationTable, dMDBMappingTable);

            //Connect to data model database
            dMDBImporter->connect();

            //Initialize instrument vector from data model database parameters
            Instrument::SPVector m_instrumentList_spvector =
                dMDBImporter->retrieveInstrumentList(instrumentList);

            Instrument::SP m_defaultInstrument_sp;

            //Initialize default instrument from data model database parameters
            if(!defaultInstrument.empty())
                m_defaultInstrument_sp =
                    dMDBImporter->retrieveInstrument(defaultInstrument);

            //Disconnect
            dMDBImporter->disconnect();

            //Create event thread
            m_eventThread_sp = EventThread::create(this, m_configuration_sp,
                m_instrumentList_spvector, m_defaultInstrument_sp);

            //Start device if auto start enabled
            if(autoStart)
            {
                INFO_STREAM << "FitsImporter::init_device() auto start enabled " << endl;
                on();
            }
        }
        catch(std::exception& ex)
        {
            set_state(Tango::FAULT);
            std::stringstream error_stream;
            error_stream << "FitsImporter::init_device() " << ex.what() << std::endl;
            set_status(error_stream.str());
        }
        catch(...)
        {
            set_state(Tango::FAULT);
            set_status("FitsImporter::init_device() unknown error");
        }
    }

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::init_device
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void FitsImporter::get_device_property()
{
	/*----- PROTECTED REGION ID(FitsImporter::get_device_property_before) ENABLED START -----*/

	//	Initialize property data members

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("WatchPath"));
	dev_prop.push_back(Tango::DbDatum("WorkerNumber"));
	dev_prop.push_back(Tango::DbDatum("ConnectionNumber"));
	dev_prop.push_back(Tango::DbDatum("EventList"));
	dev_prop.push_back(Tango::DbDatum("SleepTime"));
	dev_prop.push_back(Tango::DbDatum("WaitTime"));
	dev_prop.push_back(Tango::DbDatum("InstrumentList"));
	dev_prop.push_back(Tango::DbDatum("DefaultInstrument"));
	dev_prop.push_back(Tango::DbDatum("DMDBHost"));
	dev_prop.push_back(Tango::DbDatum("DMDBPort"));
	dev_prop.push_back(Tango::DbDatum("DMDBUser"));
	dev_prop.push_back(Tango::DbDatum("DMDBPassword"));
	dev_prop.push_back(Tango::DbDatum("DMDBSchema"));
	dev_prop.push_back(Tango::DbDatum("DMDBInstrumentTable"));
	dev_prop.push_back(Tango::DbDatum("DMDBDestinationTable"));
	dev_prop.push_back(Tango::DbDatum("DMDBMappingTable"));
	dev_prop.push_back(Tango::DbDatum("AutoStart"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);

		//	get instance on FitsImporterClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		FitsImporterClass	*ds_class =
			(static_cast<FitsImporterClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize WatchPath from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  watchPath;
		else {
			//	Try to initialize WatchPath from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  watchPath;
		}
		//	And try to extract WatchPath value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  watchPath;

		//	Try to initialize WorkerNumber from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  workerNumber;
		else {
			//	Try to initialize WorkerNumber from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  workerNumber;
		}
		//	And try to extract WorkerNumber value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  workerNumber;

		//	Try to initialize ConnectionNumber from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  connectionNumber;
		else {
			//	Try to initialize ConnectionNumber from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  connectionNumber;
		}
		//	And try to extract ConnectionNumber value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  connectionNumber;

		//	Try to initialize EventList from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  eventList;
		else {
			//	Try to initialize EventList from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  eventList;
		}
		//	And try to extract EventList value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  eventList;

		//	Try to initialize SleepTime from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  sleepTime;
		else {
			//	Try to initialize SleepTime from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  sleepTime;
		}
		//	And try to extract SleepTime value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  sleepTime;

		//	Try to initialize WaitTime from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  waitTime;
		else {
			//	Try to initialize WaitTime from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  waitTime;
		}
		//	And try to extract WaitTime value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  waitTime;

		//	Try to initialize InstrumentList from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  instrumentList;
		else {
			//	Try to initialize InstrumentList from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  instrumentList;
		}
		//	And try to extract InstrumentList value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  instrumentList;

		//	Try to initialize DefaultInstrument from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  defaultInstrument;
		else {
			//	Try to initialize DefaultInstrument from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  defaultInstrument;
		}
		//	And try to extract DefaultInstrument value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultInstrument;

		//	Try to initialize DMDBHost from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBHost;
		else {
			//	Try to initialize DMDBHost from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBHost;
		}
		//	And try to extract DMDBHost value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBHost;

		//	Try to initialize DMDBPort from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBPort;
		else {
			//	Try to initialize DMDBPort from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBPort;
		}
		//	And try to extract DMDBPort value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBPort;

		//	Try to initialize DMDBUser from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBUser;
		else {
			//	Try to initialize DMDBUser from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBUser;
		}
		//	And try to extract DMDBUser value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBUser;

		//	Try to initialize DMDBPassword from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBPassword;
		else {
			//	Try to initialize DMDBPassword from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBPassword;
		}
		//	And try to extract DMDBPassword value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBPassword;

		//	Try to initialize DMDBSchema from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBSchema;
		else {
			//	Try to initialize DMDBSchema from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBSchema;
		}
		//	And try to extract DMDBSchema value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBSchema;

		//	Try to initialize DMDBInstrumentTable from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBInstrumentTable;
		else {
			//	Try to initialize DMDBInstrumentTable from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBInstrumentTable;
		}
		//	And try to extract DMDBInstrumentTable value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBInstrumentTable;

		//	Try to initialize DMDBDestinationTable from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBDestinationTable;
		else {
			//	Try to initialize DMDBDestinationTable from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBDestinationTable;
		}
		//	And try to extract DMDBDestinationTable value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBDestinationTable;

		//	Try to initialize DMDBMappingTable from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dMDBMappingTable;
		else {
			//	Try to initialize DMDBMappingTable from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dMDBMappingTable;
		}
		//	And try to extract DMDBMappingTable value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dMDBMappingTable;

		//	Try to initialize AutoStart from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  autoStart;
		else {
			//	Try to initialize AutoStart from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  autoStart;
		}
		//	And try to extract AutoStart value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  autoStart;

	}

	/*----- PROTECTED REGION ID(FitsImporter::get_device_property_after) ENABLED START -----*/

    try
    {
        if(watchPath.empty())
            throw(invalid_argument("WatchPath property is empty or not defined"));

        if(workerNumber<1 || workerNumber>MAX_WORKER_NUMBER)
            throw(invalid_argument("WorkerNumber property out of range or not defined"));

        if(connectionNumber<1 || connectionNumber>MAX_CONNECTION_NUMBER)
            throw(invalid_argument("ConnectionNumber property out of range or not defined"));

        if(eventList.empty())
            throw(invalid_argument("EventList property is empty or not defined"));

        for(vector<string>::size_type e_ind=0; e_ind<eventList.size(); ++e_ind)
            if(eventList.at(e_ind).empty())
            {
                stringstream event_list_error;
                event_list_error << "EventList property has an empty element at \""
                        << e_ind << "\" position" << endl;
                throw(invalid_argument(event_list_error.str()));
            }

        if(sleepTime<MIN_SLEEP_TIME || sleepTime>MAX_SLEEP_TIME)
            throw(invalid_argument("SleepTime property is out of range or not defined"));

        if(waitTime>MAX_WAIT_TIME)
            throw(invalid_argument("WaitTime property is out of range or not defined"));

        if(instrumentList.empty())
            throw(invalid_argument("InstrumentList property is empty or not defined"));

        for(vector<string>::size_type i_ind=0; i_ind<instrumentList.size(); ++i_ind)
            if(instrumentList.at(i_ind).empty())
            {
                stringstream event_list_error;
                event_list_error << "InstrumentList property has an empty element at \""
                        << i_ind << "\" position" << endl;
                throw(invalid_argument(event_list_error.str()));
            }

        if(defaultInstrument.empty())
            WARN_STREAM << "DefaultInstrument property is empty or not defined" << endl;

        if(dMDBHost.empty())
            throw(invalid_argument("DMDBHost property is empty or not defined"));

        if(dMDBPort<1 || dMDBPort>MAX_DMDB_PORT)
            throw(invalid_argument("DMDB property out of range or not defined"));

        if(dMDBUser.empty())
            throw(invalid_argument("DMDBUser property is empty or not defined"));

        if(dMDBPassword.empty())
            throw(invalid_argument("DMDBPassword property is empty or not defined"));

        if(dMDBSchema.empty())
            throw(invalid_argument("DMDBSchema property is empty or not defined"));

        if(dMDBInstrumentTable.empty())
            throw(invalid_argument("DMDBInstrumentTable property is empty or not defined"));

        if(dMDBDestinationTable.empty())
            throw(invalid_argument("DMDBDestinationTable property is empty or not defined"));

        if(dMDBMappingTable.empty())
            throw(invalid_argument("DMDBMappingTable property is empty or not defined"));
    }
    catch(invalid_argument& ex)
    {
        set_state(Tango::FAULT);
        stringstream error_stream;
        error_stream << "FitsImporter::get_device_property() " << ex.what() << endl;
        set_status(error_stream.str());
    }

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void FitsImporter::always_executed_hook()
{
	DEBUG_STREAM << "FitsImporter::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::always_executed_hook) ENABLED START -----*/

	if(get_state() != Tango::FAULT)
    {
        if(m_eventThread_sp)
        {
            set_state(m_eventThread_sp->readState());

            set_status(m_eventThread_sp->readStatus());
        }
    }

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void FitsImporter::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "FitsImporter::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(FitsImporter::read_attr_hardware) ENABLED START -----*/

	//	Add your own code

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::read_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute RegularFileCounter related method
 *	Description: Indicate file correctly ingested
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void FitsImporter::read_RegularFileCounter(Tango::Attribute &attr)
{
	DEBUG_STREAM << "FitsImporter::read_RegularFileCounter(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(FitsImporter::read_RegularFileCounter) ENABLED START -----*/

    boost::mutex::scoped_lock regularCounterLock(m_regularCounterMutex);

	attr.set_value(attr_RegularFileCounter_read);

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::read_RegularFileCounter
}
//--------------------------------------------------------
/**
 *	Read attribute WarningFileCounter related method
 *	Description: Indicate file ingested in default instrument
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void FitsImporter::read_WarningFileCounter(Tango::Attribute &attr)
{
	DEBUG_STREAM << "FitsImporter::read_WarningFileCounter(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(FitsImporter::read_WarningFileCounter) ENABLED START -----*/

    boost::mutex::scoped_lock warningCounterLock(m_warningCounterMutex);

	attr.set_value(attr_WarningFileCounter_read);

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::read_WarningFileCounter
}
//--------------------------------------------------------
/**
 *	Read attribute ErrorFileCounter related method
 *	Description: Indicate file not ingested due to an error
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void FitsImporter::read_ErrorFileCounter(Tango::Attribute &attr)
{
	DEBUG_STREAM << "FitsImporter::read_ErrorFileCounter(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(FitsImporter::read_ErrorFileCounter) ENABLED START -----*/

    boost::mutex::scoped_lock errorCounterLock(m_errorCounterMutex);

	attr.set_value(attr_ErrorFileCounter_read);

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::read_ErrorFileCounter
}

//--------------------------------------------------------
/**
 *	Method      : FitsImporter::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void FitsImporter::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(FitsImporter::add_dynamic_attributes) ENABLED START -----*/

	//	Add your own code to create and add dynamic attributes if any

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command On related method
 *	Description: Activate fits importer
 *
 */
//--------------------------------------------------------
void FitsImporter::on()
{
	DEBUG_STREAM << "FitsImporter::On()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::on) ENABLED START -----*/

	try
    {
        if(m_eventThread_sp)
            m_eventThread_sp->start();
    }
    catch(std::exception& ex)
    {
        set_state(Tango::ALARM);
        std::stringstream error_stream;
        error_stream << "FitsImporter::On() " << ex.what() << std::endl;
        set_status(error_stream.str());
    }
    catch(...)
    {
        set_state(Tango::ALARM);
        set_status("FitsImporter::On() unknown error");
    }

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::on
}
//--------------------------------------------------------
/**
 *	Command Off related method
 *	Description: Deactivate fits importer
 *
 */
//--------------------------------------------------------
void FitsImporter::off()
{
	DEBUG_STREAM << "FitsImporter::Off()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::off) ENABLED START -----*/

	try
    {
        if(m_eventThread_sp)
            m_eventThread_sp->stop();
    }
    catch(std::exception& ex)
    {
        set_state(Tango::ALARM);
        std::stringstream error_stream;
        error_stream << "FitsImporter::Off() " << ex.what() << std::endl;
        set_status(error_stream.str());
    }
    catch(...)
    {
        set_state(Tango::ALARM);
        set_status("FitsImporter::Off() unknown error");
    }

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::off
}
//--------------------------------------------------------
/**
 *	Command ResetCounter related method
 *	Description:
 *
 */
//--------------------------------------------------------
void FitsImporter::reset_counter()
{
	DEBUG_STREAM << "FitsImporter::ResetCounter()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(FitsImporter::reset_counter) ENABLED START -----*/

    //Reset regular file counter
    boost::mutex::scoped_lock regularCounterLock(m_regularCounterMutex);

	*attr_RegularFileCounter_read = 0;

    //Reset warning file counter
    boost::mutex::scoped_lock warningCounterLock(m_warningCounterMutex);

    *attr_WarningFileCounter_read = 0;

    //Reset error file counter
    boost::mutex::scoped_lock errorCounterLock(m_errorCounterMutex);

    *attr_ErrorFileCounter_read = 0;

	/*----- PROTECTED REGION END -----*/	//	FitsImporter::reset_counter
}

/*----- PROTECTED REGION ID(FitsImporter::namespace_ending) ENABLED START -----*/

//==============================================================================
//	FitsImporter::incrementRegularCounter()
//==============================================================================
void FitsImporter::incrementRegularCounter()
{
    DEBUG_STREAM << "FitsImporter::incrementRegularCounter()  - " << device_name << endl;

    boost::mutex::scoped_lock regularCounterLock(m_regularCounterMutex);

	++*attr_RegularFileCounter_read;
}

//==============================================================================
//	FitsImporter::incrementWarningCounter()
//==============================================================================
void FitsImporter::incrementWarningCounter()
{
    DEBUG_STREAM << "FitsImporter::incrementWarningCounter()  - " << device_name << endl;

    boost::mutex::scoped_lock warningCounterLock(m_warningCounterMutex);

    ++*attr_WarningFileCounter_read;
}

//==============================================================================
//	FitsImporter::incrementErrorCounter()
//==============================================================================
void FitsImporter::incrementErrorCounter()
{
    DEBUG_STREAM << "FitsImporter::incrementErrorCounter()  - " << device_name << endl;

    boost::mutex::scoped_lock errorCounterLock(m_errorCounterMutex);

    ++*attr_ErrorFileCounter_read;
}

//==============================================================================
//	FitsImporter::create_inotify_mask()
//==============================================================================
uint32_t FitsImporter::create_inotify_mask(const std::vector<std::string>& event_list)
    throw(std::invalid_argument)
{
    DEBUG_STREAM << "FitsImporter::create_inotify_mask()  - " << device_name << endl;

	uint32_t iNotifyMask = 0;

	if(event_list.empty())
		throw std::invalid_argument("FitsImporter::create_inotify_mask(): "
                "event list is empty");

	std::vector<std::string>::const_iterator it;
	for(it=event_list.begin(); it!=event_list.end(); it++)
	{
        std::stringstream event_stream;
        event_stream << "FitsImporter::create_inotify_mask() ";

		if(it->compare("IN_ACCESS")==0)
		{
            event_stream << "IN_ACCESS event found";
			iNotifyMask += IN_ACCESS;
		}
		else if(it->compare("IN_MODIFY")==0)
		{

            event_stream << "IN_MODIFY event found";
			iNotifyMask += IN_MODIFY;
		}
		else if(it->compare("IN_ATTRIB")==0)
		{
            event_stream << "IN_ATTRIB event found";
			iNotifyMask += IN_ATTRIB;
		}
		else if(it->compare("IN_CLOSE_WRITE")==0)
		{
            event_stream << "IN_CLOSE_WRITE event found";
			iNotifyMask += IN_CLOSE_WRITE;
		}
		else if(it->compare("IN_CLOSE_NOWRITE")==0)
		{
            event_stream << "IN_CLOSE_NOWRITE event found";
			iNotifyMask += IN_CLOSE_NOWRITE;
		}
		else if(it->compare("IN_OPEN")==0)
		{
            event_stream << "IN_OPEN event found";
			iNotifyMask += IN_OPEN;
		}
		else if(it->compare("IN_MOVED_FROM")==0)
		{
            event_stream << "IN_MOVED_FROM event found";
			iNotifyMask += IN_MOVED_FROM;
		}
		else if(it->compare("IN_MOVED_TO")==0)
		{
            event_stream << "IN_MOVED_TO event found";
			iNotifyMask += IN_MOVED_TO;
		}
		else if(it->compare("IN_DELETE")==0)
		{
            event_stream << "IN_DELETE event found";
			iNotifyMask += IN_DELETE;
		}
		else if(it->compare("IN_DELETE_SELF")==0)
		{
            event_stream << "IN_DELETE_SELF event found";
			iNotifyMask += IN_DELETE_SELF;
		}
		else if(it->compare("IN_CLOSE")==0)
		{
            event_stream << "IN_CLOSE event found";
			iNotifyMask += IN_CLOSE;
		}
		else if(it->compare("IN_MOVE")==0)
		{
            event_stream << "IN_MOVE event found";
			iNotifyMask += IN_MOVE;
		}
		else if(it->compare("IN_ALL_EVENTS")==0)
		{
            event_stream << "IN_ALL_EVENTS event found";
			iNotifyMask += IN_ALL_EVENTS;
		}
		else
			throw std::invalid_argument("FitsImporter::create_inotify_mask() "
                    "string \"" + *it + " \" is invalid inotify event");

        INFO_STREAM << event_stream.str() << endl;
	}

	return iNotifyMask;
}

/*----- PROTECTED REGION END -----*/	//	FitsImporter::namespace_ending
} //	namespace
