/*----- PROTECTED REGION ID(FitsImporter.h) ENABLED START -----*/
//=============================================================================
//
// file :        FitsImporter.h
//
// description : Include file for the FitsImporter class
//
// project :     Fits importer device server
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
// $Author:  $
//
// $Revision:  $
// $Date:  $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef FitsImporter_H
#define FitsImporter_H

#include <EventThread.h>

#include <tango.h>

#include <boost/thread.hpp>

/*----- PROTECTED REGION END -----*/	//	FitsImporter.h

/**
 *  FitsImporter class description:
 *    Fits importer device server
 */

namespace FitsImporter_ns
{
/*----- PROTECTED REGION ID(FitsImporter::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations

/*----- PROTECTED REGION END -----*/	//	FitsImporter::Additional Class Declarations

class FitsImporter : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(FitsImporter::Data Members) ENABLED START -----*/

//------------------------------------------------------------------------------
//  [Private] Class variables
//------------------------------------------------------------------------------
    //Thread shared pointer
    EventThread::SP m_eventThread_sp;

    //Regular file counter synchronization
	boost::mutex m_regularCounterMutex;

    //Warning file counter synchronization
	boost::mutex m_warningCounterMutex;

    //Error file counter synchronization
    boost::mutex m_errorCounterMutex;

    //Max number of worker thread allowed
    static const unsigned int MAX_WORKER_NUMBER = 100;

    //Max number of connection per database allowed
    static const unsigned int MAX_CONNECTION_NUMBER = 10;

    //Min milli second of sleep time allowed
    static const unsigned long MIN_SLEEP_TIME = 100;

    //Max milli second of sleep time allowed
    static const unsigned long MAX_SLEEP_TIME = 10000;

    //Max milli second of wait time allowed
    static const unsigned long MAX_WAIT_TIME = 10000;

    //Max port number allowed value
    static const unsigned int MAX_DMDB_PORT = 65535;

/*----- PROTECTED REGION END -----*/	//	FitsImporter::Data Members

//	Device property data members
public:
	//	WatchPath:	Absolute path where fits importer wait for a new file to ingest
	string	watchPath;
	//	WorkerNumber:	Number of worker thread to process new file
	Tango::DevUShort	workerNumber;
	//	ConnectionNumber:	Number of connection created in connection pool for each database entry in destination table
	Tango::DevUShort	connectionNumber;
	//	EventList:	Inotify event that trigger fits importer new file event.
	//  Allowed event are:
	//  * IN_ACCESS
	//  * IN_MODIFY
	//  * IN_ATTRIB
	//  * IN_CLOSE_WRITE
	//  * IN_CLOSE_NOWRITE
	//  * IN_OPEN
	//  * IN_MOVED_FROM
	//  * IN_MOVED_TO
	//  * IN_DELETE
	//  * IN_DELETE_SELF
	//  * IN_CLOSE
	//  * IN_MOVE
	//  * IN_ALL_EVENTS
	vector<string>	eventList;
	//	SleepTime:	Time fits importer event loop has to sleep before check for new file
	Tango::DevULong	sleepTime;
	//	WaitTime:	Time fits importer has to sleep between new time event and ingestion
	Tango::DevULong	waitTime;
	//	InstrumentList:	List of instrument used by fits importer when ingesting file
	vector<string>	instrumentList;
	//	DefaultInstrument:	Default instrument used by fits importer when:
	//  1) no instrument match with file
	//  2) an error occurs during metadata extraction
	string	defaultInstrument;
	//	DMDBHost:	Hostname where data model database is running
	string	dMDBHost;
	//	DMDBPort:	Port where data model database is listening
	Tango::DevULong	dMDBPort;
	//	DMDBUser:	User to login in data model database
	string	dMDBUser;
	//	DMDBPassword:	Password to login in data model database
	string	dMDBPassword;
	//	DMDBSchema:	Schema where data model tables are located
	string	dMDBSchema;
	//	DMDBInstrumentTable:	Instrument table name
	string	dMDBInstrumentTable;
	//	DMDBDestinationTable:	Destination table name
	string	dMDBDestinationTable;
	//	DMDBMappingTable:	Mapping table name
	string	dMDBMappingTable;
	//	AutoStart:	Exec On command after init if state is not fault
	Tango::DevBoolean	autoStart;

//	Attribute data members
public:
	Tango::DevULong	*attr_RegularFileCounter_read;
	Tango::DevULong	*attr_WarningFileCounter_read;
	Tango::DevULong	*attr_ErrorFileCounter_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	FitsImporter(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	FitsImporter(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	FitsImporter(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~FitsImporter() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : FitsImporter::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute RegularFileCounter related methods
 *	Description: Indicate file correctly ingested
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
	virtual void read_RegularFileCounter(Tango::Attribute &attr);
	virtual bool is_RegularFileCounter_allowed(Tango::AttReqType type);
/**
 *	Attribute WarningFileCounter related methods
 *	Description: Indicate file ingested in default instrument
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
	virtual void read_WarningFileCounter(Tango::Attribute &attr);
	virtual bool is_WarningFileCounter_allowed(Tango::AttReqType type);
/**
 *	Attribute ErrorFileCounter related methods
 *	Description: Indicate file not ingested due to an error
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
	virtual void read_ErrorFileCounter(Tango::Attribute &attr);
	virtual bool is_ErrorFileCounter_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : FitsImporter::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();



//	Command related methods
public:
	/**
	 *	Command On related method
	 *	Description: Activate fits importer
	 *
	 */
	virtual void on();
	virtual bool is_On_allowed(const CORBA::Any &any);
	/**
	 *	Command Off related method
	 *	Description: Deactivate fits importer
	 *
	 */
	virtual void off();
	virtual bool is_Off_allowed(const CORBA::Any &any);
	/**
	 *	Command ResetCounter related method
	 *	Description:
	 *
	 */
	virtual void reset_counter();
	virtual bool is_ResetCounter_allowed(const CORBA::Any &any);


/*----- PROTECTED REGION ID(FitsImporter::Additional Method prototypes) ENABLED START -----*/

//------------------------------------------------------------------------------
//  [Public] Users methods
//------------------------------------------------------------------------------
    virtual void incrementRegularCounter();

    virtual void incrementWarningCounter();

    virtual void incrementErrorCounter();

private:
//------------------------------------------------------------------------------
//  [Private] Utilities methods
//------------------------------------------------------------------------------
    uint32_t create_inotify_mask(const std::vector<std::string>&)
        throw (std::invalid_argument);

/*----- PROTECTED REGION END -----*/	//	FitsImporter::Additional Method prototypes
};

/*----- PROTECTED REGION ID(FitsImporter::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	FitsImporter::Additional Classes Definitions

}	//	End of namespace

#endif   //	FitsImporter_H
