#ifndef INSTRUMENT_H
#define INSTRUMENT_H

#include <iostream>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <Destination.h>

namespace FitsImporter_ns
{

class Instrument
{
public:
//------------------------------------------------------------------------------
//	[Public] Shared pointer typedef	
//------------------------------------------------------------------------------
	typedef boost::shared_ptr<Instrument> SP;
	typedef std::vector< SP > SPVector;

private:
//------------------------------------------------------------------------------
//	[Private] Constructor destructor deleter
//------------------------------------------------------------------------------
	Instrument(std::string name, int fitsKeyHdu, std::string fitsKey, std::string fitsValue,
		int fitsDateHdu, std::string fitsDate, boost::shared_ptr<Destination> destination_sp) :		
		m_name(name), m_fitsKeyHdu(fitsKeyHdu), m_fitsKey(fitsKey), m_fitsValue(fitsValue), 
		m_fitsDateHdu(fitsDateHdu), m_fitsDate(fitsDate), m_destination_sp(destination_sp) {}		
	~Instrument() {}

	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(Instrument* i) { delete i; }
	};

public:
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------
	static Instrument::SP create(std::string name, int fitsKeyHdu, std::string fitsKey,
		std::string fitsValue, int fitsDateHdu, std::string fitsDate, Destination::SP destination_sp)		
	{
		Instrument::SP d_sp( new Instrument(name, fitsKeyHdu, fitsKey, fitsValue, fitsDateHdu, fitsDate,
			 destination_sp), Instrument::Deleter() );
		return d_sp;
	}

	std::string getName() const { return m_name; }
	int getFitsKeyHdu() const { return m_fitsKeyHdu; }
	std::string getFitsKey() const { return m_fitsKey; }
	std::string getFitsValue() const { return m_fitsValue; }
	int getFitsDateHdu() const { return m_fitsDateHdu; }
	std::string getFitsDate() const { return m_fitsDate; }
	boost::shared_ptr<Destination> getDestination() { return m_destination_sp; }

private:
//------------------------------------------------------------------------------
//	[Private] Class variables
//------------------------------------------------------------------------------    
	//Instrument name
	const std::string m_name;
	
	//Instrument fits key hdu 
	const int m_fitsKeyHdu;

	//Instrument fits key 
	const std::string m_fitsKey;

	//Instrument fits value
	const std::string m_fitsValue;

	//Date fits key hdu
	const int m_fitsDateHdu;
	
	//Date fits key
	const std::string m_fitsDate;
	
	//Destination of instrument meta data
	Destination::SP m_destination_sp;
};

}   //End of namespace

#endif /*!INSTRUMENT_H*/
