#ifndef MAPPING_H
#define MAPPING_H

#include <iostream>
#include <vector>
#include <boost/shared_ptr.hpp>

namespace FitsImporter_ns
{

class Mapping
{
public:
//------------------------------------------------------------------------------
//	[Public] Shared pointer typedef
//------------------------------------------------------------------------------
	typedef boost::shared_ptr<Mapping> SP;
	typedef std::vector< Mapping::SP > SPVector;

private:
//------------------------------------------------------------------------------
//	[Private] Constructor destructor deleter
//------------------------------------------------------------------------------
	Mapping(std::string columnName, std::string columnType, int fitsHDU,
		std::string fitsPrimary, std::string fitsSecondary, bool mandatory) :
		m_columnName(columnName), m_columnType(columnType), m_fitsHDU(fitsHDU),
		m_fitsPrimary(fitsPrimary), m_fitsSecondary(fitsSecondary),
        m_mandatory(mandatory) {}
	~Mapping() {}

	class Deleter;
	friend class Deleter;
	class Deleter
	{
	public:
		void operator()(Mapping* m) { delete m; }
	};

public:
//------------------------------------------------------------------------------
//	[Public] Users methods
//------------------------------------------------------------------------------
	static Mapping::SP create(std::string columnName, std::string columnType,
	 	int fitsHDU, std::string fitsPrimary,
		std::string fitsSecondary, bool mandatory)
	{
		Mapping::SP m_sp( new Mapping( columnName, columnType, fitsHDU,
            fitsPrimary, fitsSecondary, mandatory), Mapping::Deleter());
		return m_sp;
	}

	std::string getColumnName() const { return m_columnName; }
	std::string getColumnType() const { return m_columnType; }
    int getFitsHDU() const { return m_fitsHDU; }
	std::string getFitsPrimary() const { return m_fitsPrimary; }
	std::string getFitsSecondary() const { return m_fitsSecondary; }
	bool isMandatory() const { return m_mandatory; }

private:
//------------------------------------------------------------------------------
//	[Private] Class variables
//------------------------------------------------------------------------------
	//Metadata table column name
	const std::string m_columnName;

	//Metadata table column type
	const std::string m_columnType;

    //Metadata fits primary and secondary keys HDU
    const int m_fitsHDU;

	//Metadata fits primary key to be stored
	const std::string m_fitsPrimary;

	//Metadata fits secondary key to be stored
	const std::string m_fitsSecondary;

	//Metadata fits key mandatory flags
	const bool m_mandatory;
};

}   //End of namespace

#endif /*!MAPPING_H*/
