#ifndef WORKER_THREAD_H
#define WORKER_THREAD_H

#include <Configuration.h>
#include <Instrument.h>
#include <EventBuffer.h>
#include <ConnectionManager.h>

#include <tango.h>

#include <stdexcept>
#include <sstream>

#include <CCfits/CCfits>

#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>
#include <boost/filesystem.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/date_clock_device.hpp>

namespace FitsImporter_ns
{

class FitsImporter;

class WorkerThread : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//	[Public] Constructor destructor
//------------------------------------------------------------------------------
	WorkerThread(FitsImporter*, EventBuffer::SP, ConnectionManager::SP,
        Configuration::SP, Instrument::SPVector, Instrument::SP);
	virtual ~WorkerThread();

//------------------------------------------------------------------------------
//	[Public] Thread loop method
//------------------------------------------------------------------------------
	virtual void workerLoop();

protected:
//------------------------------------------------------------------------------
//	[Protected] Instrument related archiving process methods
//------------------------------------------------------------------------------
    virtual void ingestUsingInstrumentList(boost::filesystem::path&,
        boost::shared_ptr<CCfits::FITS>, std::set<int>&)
        throw(CCfits::FitsException, std::runtime_error);

    virtual void ingestUsingDefaultInstrument(boost::filesystem::path&,
        boost::shared_ptr<CCfits::FITS>, std::set<int>&)
        throw(CCfits::FitsException, std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] Date parsing methods
//------------------------------------------------------------------------------
	virtual boost::gregorian::date parseDate(std::string)
		throw(boost::bad_lexical_cast, std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] Common archiving process method
//------------------------------------------------------------------------------
    virtual void processFits(boost::filesystem::path&,
        boost::shared_ptr<CCfits::FITS>, std::set<int>&, Instrument::SP,
        boost::gregorian::date&) throw(CCfits::FitsException, std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] Duplicate discovery method
//------------------------------------------------------------------------------
	virtual int findDuplicateMax(ConnectionManager::SessionSP,
    	     Destination::SP, boost::filesystem::path&, std::string checksum)
		throw(std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] Metadata archiving methods
//------------------------------------------------------------------------------
	virtual std::string composeQuery(Destination::SP, boost::gregorian::date&,
        int, std::string, std::string, boost::filesystem::path&, boost::shared_ptr<CCfits::FITS>,
        std::set<int>&) throw(CCfits::FitsException, std::runtime_error);

    virtual void fillQueryFromFits(std::stringstream&, std::stringstream&,
        Destination::SP, boost::shared_ptr<CCfits::FITS>, std::set<int>&)
        throw(CCfits::FitsException, std::runtime_error);

	virtual std::string readKeyFromPHDU(CCfits::PHDU&, std::string, std::string)
        throw(CCfits::FitsException, std::runtime_error);

	virtual std::string readKeyFromHDU(CCfits::HDU&, std::string, std::string)
        throw(CCfits::FitsException, std::runtime_error);

	virtual std::string readKey(CCfits::HDU&, std::string, std::string)
        throw(CCfits::FitsException, std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] File archiving methods
//------------------------------------------------------------------------------
	virtual boost::filesystem::path createDestPath(boost::filesystem::path&,
		Destination::SP, boost::gregorian::date&, int)
		throw(std::runtime_error);

	virtual void copyFile(boost::filesystem::path&, boost::filesystem::path& )
		throw(std::runtime_error);

//------------------------------------------------------------------------------
//	[Protected] Class variables
//------------------------------------------------------------------------------
    //Tango server class pointer
    FitsImporter* m_fitsImporter_p;

	//Event buffer shared pointer
	EventBuffer::SP m_eventBuffer_sp;

	//Connection manager shared_pointer
	ConnectionManager::SP m_connectionManager_sp;

	//Configuration shared pointer
	Configuration::SP m_configuration_sp;

	//Instrument list shared pointer vector
	Instrument::SPVector m_instrumentList_spvector;

    //Default instrument shared pointer
    Instrument::SP m_defaultInstrument_sp;
};

}   //End of namespace

#endif /*!WORKER_THREAD_H*/
